# Reb00t Perl RPM macros
# $Id: macros.reb00t.perl,v 1.2 2007/09/28 11:40:35 altblue Exp $

# Return the value of a %Config variable
%perl_config()   %(%{__perl} -MConfig -e 'print $Config{%1}')

# Return the value of a %Config variable in "RPM boolean" context (0/1)
%perl_has()      %(%{__perl} -MConfig -e 'print $Config{%1} ? 1 : 0')

# perl version
%perl_version    %{expand:%%{perl_config version}}

# perl directories (always use vendor* install!):
%perl_vendorarch %{expand:%%{perl_config vendorarchexp}}
%perl_vendorlib  %{expand:%%{perl_config vendorlibexp}}
%perl_sitearch   %{expand:%%{perl_config sitearchexp}}
%perl_sitelib    %{expand:%%{perl_config sitelibexp}}
%perl_archlib    %{expand:%%{perl_config archlibexp}}
%perl_privlib    %{expand:%%{perl_config privlibexp}}


# RPM version comparison is quite different to what CPAN packagers use.
# From time to time we have to tweak CPAN distribution's version into something
# RPM friendlier, so here comes this twist that offers some kind of "visual cue"
# by adding the original CPAN distro version to RPM's release:
%orig_version() %([ "%1" != "%2" ] && echo ".%1" | %{__tr} -- - _)
%cpan_version %{orig_version -- "%{pkg_version}" "%{version}"}


# Perl features requirements generator:
%perl_req_feat() %(%{__perl} -MConfig -le '             \
  foreach (split q{ }, q{%*}) {                         \
    (my $feat = uc $_) =~ s/^USE//;                     \
    print +q{Requires: perl(:WITH},                     \
      ($Config{$_} ? q{} : q{OUT}), q{_}, $feat, q{)};  \
  }                                                     \
')

# (RedHat legacy/compatibility) Require these features for arch packages:
%perl_req_arch %{perl_req_feat useithreads usethreads uselargefiles}

# Shortcut for requiring Perl @INC compatibility
# (as "perl" package provides)
%perl_req_compat                               \
Requires: perl(:MODULE_COMPAT_%{perl_version}) \
%ifnarch noarch                                \
%perl_req_arch                                 \
%endif


# Fix perl binary path in scripts (e.g. #!/usr/local/bin/perl)
%fix_perl_path() %{__grep} -rsl '^#!.*perl' %* \\\
  | %{__xargs} -r %{__perl} -MExtUtils::MakeMaker -e 'MY->fixin(@ARGV)'


# Some Module::Build based distros miss ExtUtils::MakeMaker compatible
# Makefile.PL. This fixes it.
# - please keep it Perl::Critic/Tidy "safe" ;-)
%make_makefile_pl\
%{__cat} << 'EOF' > Makefile.PL\
use Module::Build::Compat;\
Module::Build::Compat->run_build_pl( args => \\@ARGV );\
Module::Build::Compat->write_makefile();\
EOF


# Basic perl distro building procedure:
%perl_build()                                                             \
unset PERL_UNICODE PERL5LIB PERL5OPT PERLIO PERLLIB PERL5DB               \
export PERL_MM_USE_DEFAULT=1                                              \
%ifarch noarch                                                            \
%{__perl} Makefile.PL INSTALLDIRS=vendor %{*} < /dev/null                 \
%{__make} < /dev/null                                                     \
%else                                                                     \
export CFLAGS="$RPM_OPT_FLAGS"                                            \
%{__perl} Makefile.PL INSTALLDIRS=vendor %{*} < /dev/null                 \
%{__find} . -name Makefile -print0 | %{__xargs} -0r                     \\\
%{__perl} -pi -e "s/^(\\s*)LD_RUN_PATH=(?:(['\\"]).+?\\2|\\S+)\\s+/\\$1/" \
%{__make} %{?_smp_mflags} OPTIMIZE="$RPM_OPT_FLAGS" < /dev/null           \
%endif


# Basic perl distro installation procedure:
# -f : full install ("make install")
# -o : use old/obsolete hook ("PERL_INSTALL_ROOT") instead DESTDIR
%perl_install(of)                                                        \
%{!-f:%{__make} pure_install}%{-f:%{makeinstall}}                      \\\
    %{-o:PERL_INSTALL_ROOT}%{!-o:DESTDIR}=%{buildroot} %{*}              \
%ifnarch noarch                                                          \
%{__find} %{buildroot} -type f -name '*.bs' -a -empty -delete || :       \
%{__find} %{buildroot}%{perl_vendorarch} -name '*.so'                  \\\
                                         -exec %{__chmod} 0755 {} \\;    \
%endif                                                                   \
%{-f:%{__rm} -f %{buildroot}%{perl_archlib}/perllocal.pod}               \
%{__find} %{buildroot} -name .packlist -delete || :                      \
%{__find} %{buildroot} -depth -type d -a -empty -delete || :             \
%{__chmod} -R u+w,go-w %{buildroot}/*


# Standard files:
# -l : package uses AutoLoader/AutoSplit (has files in vendorlib/auto/...)
# -b : package has "binaries" (files in %_bindir)
# -m : package has "section-1" man pages
%perl_files(bml)                                                         \
%ifarch noarch                                                           \
%{perl_vendorlib}/*                                                      \
%{-l:%%exclude %%dir %{perl_vendorlib}/auto}                             \
%else                                                                    \
%{perl_vendorarch}/*                                                     \
%exclude %dir %{perl_vendorarch}/auto                                    \
%endif                                                                   \
%{_mandir}/man3/*.3pm.*                                                  \
%{-b:%{_bindir}/*}                                                       \
%{-m:%{_mandir}/man1/*.1.*}                                              \
%{nil}


# custom "requires" script - trigger it only for "reb00t perl" packages
%__make_perl_requires \
%{__cat} << EOF > blib/rpm-filter-req\
#!%{_buildshell}\
%{__grep} -v %{_docdir} - \\\
| %{__perl_requires} $* \\\
| %{__grep} -v '^perl >= 0:' \\\
%{?perl_filter_req:| %{__grep} -Ev %{perl_filter_req}}\
if [[ "$RPM_PACKAGE_NAME" =~ '^perl-' ]] ; then\
  rpmquery --provides -f %{__perl} | %{__grep} '^perl(:'\
fi\
EOF\
%global __perl_requires %{_builddir}/%{buildsubdir}/blib/rpm-filter-req\
%{__chmod} +x %{__perl_requires}\
%{nil}


# Custom "provides" script - trigger it only for "reb00t perl" packages
%__make_perl_provides \
%{__cat} << EOF > blib/rpm-filter-prov\
#!%{_buildshell}\
%{__grep} -v %{_docdir} - \\\
| %{__perl_provides} $* \\\
%{?perl_filter_prov:| %{__grep} -Ev %{perl_filter_prov}}\
EOF\
%global __perl_provides %{_builddir}/%{buildsubdir}/blib/rpm-filter-prov\
%{__chmod} +x %{__perl_provides}\
%{nil}


%__spec_prep_post \
if [ -f Makefile.PL -o -f Build.PL ] ; then\
if [ ! -f Makefile.PL ] ; then\
%make_makefile_pl\
fi\
%{__mkdir_p} blib\
%__make_perl_requires\
%__make_perl_provides\
fi\
%{nil}
